package jalview.datamodel;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator over each element in a set of ranges i.e. if ranges is {[3,6],
 * [12,15]} it will iterate over {3,4,5,6,12,13,14,15}. Uses a local copy of the
 * set of ranges.
 * 
 * @author kmourao
 *
 */
public class RangeElementsIterator implements Iterator<Integer>
{
  private int last;

  private int current;

  private int next;

  private Iterator<int[]> rangeIterator;

  private int[] nextRange = null;

  RangeElementsIterator(Iterator<int[]> it)
  {
    rangeIterator = it;
    if (rangeIterator.hasNext())
    {
      nextRange = rangeIterator.next();
      next = nextRange[0];
      last = nextRange[1];
    }
  }

  @Override
  public boolean hasNext()
  {
    return rangeIterator.hasNext() || next <= last;
  }

  @Override
  public Integer next()
  {
    if (!hasNext())
    {
      throw new NoSuchElementException();
    }

    current = next;

    // recalculate next
    next++;

    // if there are more ranges need to check if next is in a range
    checkNextRange();
    return current;
  }

  /**
   * Check how next position relates to next range, and update next position if
   * necessary
   */
  private void checkNextRange()
  {
    if (nextRange != null && next > nextRange[1])
    {
      if (rangeIterator.hasNext())
      {
        nextRange = rangeIterator.next();
        next = nextRange[0];
        last = nextRange[1];
      }
      else
      {
        nextRange = null;
      }

    }
  }

  @Override
  public void remove()
  {
    throw new UnsupportedOperationException();
  }
}
